//
// Directory Flush test
//
// http://bugzilla.mozilla.org/show_bug.cgi?id=221797#c4
// 

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef _WIN32
#include <windows.h>
#include <io.h>
#define sleep(s) Sleep((s)*1000L)
#define fsync(fd) _commit(fd)
#else
#include <unistd.h>
#endif


static bool printsize(const char * name, long size, const char * text)
{
	struct stat st;
	if (stat(name, &st)) // calls FindFirstFile() on Win32
		st.st_size = -1;
	bool ok = (size == st.st_size); 
	printf("%-8s %8ld, %8ld%s\n", text, size, st.st_size, (ok?"":" !"));
	fflush(stdout);
	return ok;
}


int main()
{
	const char * name = "test.tmp";
	FILE * f = fopen(name, "wb");
	if (!f) {
		perror(name);
		return 1;
	}

	printf("Func     #written, file size reported by stat()\n");
	long size = 0;
	for (int i = 0; i < 10; i++) {
		for (int j = 0; j < 25000; j++) {
			static const char line[] = "12345678901234567890123456789012345678\r\n";
			fputs(line, f); size += sizeof(line)-1;
		}
		//printsize(name, size, "write");
		fflush(f);
		bool ok = printsize(name, size, "flush");
		if (i % 4 == 3) {
			if (fsync(fileno(f))) // calls FlushFileBuffers() on Win32
				perror("fsync");
			printsize(name, size, "*fsync*");
		}
		else if (!ok) {
			for (int j = 0; j < 20; j++) {
				sleep(1);
				if (printsize(name, size, "sleep"))
					break;
			}
		}

	}
	fclose(f);
	printsize(name, size, "close");
	return 0;
}
